@@ -1,24 +1,6 @@
 /*
- * Copyright (c) 2012 Rob Clark <robdclark@gmail.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright © 2012 Rob Clark <robdclark@gmail.com>
+ * SPDX-License-Identifier: MIT
  */
 
 #include <assert.h>
@@ -76,6 +58,7 @@ is_64b(void)
 static int draws[4];
 static struct {
    uint64_t base;
+   uint32_t *host_base;
    uint32_t size; /* in dwords */
    /* Generally cmdstream consists of multiple IB calls to different
     * buffers, which are themselves often re-used for each tile.  The
@@ -167,9 +150,9 @@ static void dump_tex_samp(uint32_t *texsamp, enum state_src_t src, int num_unit,
 static void dump_tex_const(uint32_t *texsamp, int num_unit, int level);
 
 static bool
-highlight_gpuaddr(uint64_t gpuaddr)
+highlight_addr(uint32_t *hostaddr)
 {
-   if (!options->ibs[ib].base)
+   if (!options->ibs[ib].base && (ib != 0 || !options->rb_host_base))
       return false;
 
    if ((ib > 0) && options->ibs[ib - 1].base &&
@@ -182,13 +165,17 @@ highlight_gpuaddr(uint64_t gpuaddr)
    if (ibs[ib].triggered)
       return options->color;
 
-   if (options->ibs[ib].base != ibs[ib].base)
+   if (ib != 0 && options->ibs[ib].base != ibs[ib].base)
       return false;
 
-   uint64_t start = ibs[ib].base + 4 * (ibs[ib].size - options->ibs[ib].rem);
-   uint64_t end = ibs[ib].base + 4 * ibs[ib].size;
+   uint32_t *host_base = (ib != 0) ? ibs[ib].host_base :
+      options->rb_host_base;
+   uint32_t size = options->ibs[ib].size ? options->ibs[ib].size :
+      ibs[ib].size;
+   uint32_t *start = host_base + (size - options->ibs[ib].rem);
+   uint32_t *end = host_base + size;
 
-   bool triggered = (start <= gpuaddr) && (gpuaddr <= end);
+   bool triggered = (start <= hostaddr) && (hostaddr <= end);
 
    if (triggered && (ib < 2) && options->ibs[ib + 1].crash_found) {
       ibs[ib].base_seen = true;
@@ -212,7 +199,7 @@ dump_hex(uint32_t *dwords, uint32_t sizedwords, int level)
    if (quiet(2))
       return;
 
-   bool highlight = highlight_gpuaddr(gpuaddr(dwords) + 4 * sizedwords - 1);
+   bool highlight = highlight_addr(dwords + sizedwords - 1);
 
    for (i = 0; i < sizedwords; i += 8) {
       int zero = 1;
@@ -695,34 +682,34 @@ static struct {
       REG(CP_SCRATCH[0x6].REG, reg_dump_scratch),
       REG(CP_SCRATCH[0x7].REG, reg_dump_scratch),
 
-      REG64(SP_VS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_HS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_DS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_GS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_FS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_CS_OBJ_START, reg_disasm_gpuaddr64),
+      REG64(SP_VS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_HS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_DS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_GS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_PS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_CS_BASE, reg_disasm_gpuaddr64),
 
-      REG64(SP_VS_TEX_CONST, reg_dump_gpuaddr64),
-      REG64(SP_VS_TEX_SAMP, reg_dump_gpuaddr64),
-      REG64(SP_HS_TEX_CONST, reg_dump_gpuaddr64),
-      REG64(SP_HS_TEX_SAMP, reg_dump_gpuaddr64),
-      REG64(SP_DS_TEX_CONST, reg_dump_gpuaddr64),
-      REG64(SP_DS_TEX_SAMP, reg_dump_gpuaddr64),
-      REG64(SP_GS_TEX_CONST, reg_dump_gpuaddr64),
-      REG64(SP_GS_TEX_SAMP, reg_dump_gpuaddr64),
-      REG64(SP_FS_TEX_CONST, reg_dump_gpuaddr64),
-      REG64(SP_FS_TEX_SAMP, reg_dump_gpuaddr64),
-      REG64(SP_CS_TEX_CONST, reg_dump_gpuaddr64),
-      REG64(SP_CS_TEX_SAMP, reg_dump_gpuaddr64),
+      REG64(SP_VS_TEXMEMOBJ_BASE, reg_dump_gpuaddr64),
+      REG64(SP_VS_SAMPLER_BASE, reg_dump_gpuaddr64),
+      REG64(SP_HS_TEXMEMOBJ_BASE, reg_dump_gpuaddr64),
+      REG64(SP_HS_SAMPLER_BASE, reg_dump_gpuaddr64),
+      REG64(SP_DS_TEXMEMOBJ_BASE, reg_dump_gpuaddr64),
+      REG64(SP_DS_SAMPLER_BASE, reg_dump_gpuaddr64),
+      REG64(SP_GS_TEXMEMOBJ_BASE, reg_dump_gpuaddr64),
+      REG64(SP_GS_SAMPLER_BASE, reg_dump_gpuaddr64),
+      REG64(SP_PS_TEXMEMOBJ_BASE, reg_dump_gpuaddr64),
+      REG64(SP_PS_SAMPLER_BASE, reg_dump_gpuaddr64),
+      REG64(SP_CS_TEXMEMOBJ_BASE, reg_dump_gpuaddr64),
+      REG64(SP_CS_SAMPLER_BASE, reg_dump_gpuaddr64),
 
       {NULL},
 }, reg_a7xx[] = {
-      REG64(SP_VS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_HS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_DS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_GS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_FS_OBJ_START, reg_disasm_gpuaddr64),
-      REG64(SP_CS_OBJ_START, reg_disasm_gpuaddr64),
+      REG64(SP_VS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_HS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_DS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_GS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_PS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_CS_BASE, reg_disasm_gpuaddr64),
 
       {NULL},
 }, *type0_reg;
@@ -1508,6 +1495,70 @@ dump_tex_const(uint32_t *texconst, int num_unit, int level)
    }
 }
 
+static void
+dump_bindless_descriptors(bool is_compute, int level)
+{
+   if (!options->dump_bindless)
+      return;
+
+   printl(2, "%sdraw[%i] bindless descriptors\n", levels[level], draw_count);
+
+   for (unsigned i = 0; i < 128; i++) {
+      static char reg_name[64];
+      if (is_compute) {
+         sprintf(reg_name, "SP_CS_BINDLESS_BASE[%u].DESCRIPTOR", i);
+      } else {
+         sprintf(reg_name, "SP_GFX_BINDLESS_BASE[%u].DESCRIPTOR", i);
+      }
+      const unsigned base_reg = regbase(reg_name);
+      if (!base_reg)
+         break;
+
+      printl(2, "%sset[%u]:\n", levels[level + 1], i);
+
+      uint64_t ext_src_addr;
+      if (is_64b()) {
+         const unsigned reg = base_reg + i * 2;
+         if (!reg_written(reg))
+            continue;
+
+         ext_src_addr = reg_val(reg) & 0xfffffffc;
+         ext_src_addr |= ((uint64_t)reg_val(reg + 1)) << 32;
+      } else {
+         const unsigned reg = base_reg + i;
+         if (!reg_written(reg))
+            continue;
+
+         ext_src_addr = reg_val(reg) & 0xfffffffc;
+      }
+
+      uint32_t *contents = NULL;
+      if (ext_src_addr)
+         contents = hostptr(ext_src_addr);
+
+      if (!contents)
+         continue;
+
+      uint32_t empty_contents[16] = {};
+
+      unsigned length = hostlen(ext_src_addr);
+      unsigned desc_count = length / (16 * sizeof(uint32_t));
+      for (unsigned desc_idx = 0; desc_idx < desc_count; desc_idx++) {
+         if (memcmp(contents, empty_contents, sizeof(empty_contents))) {
+            printl(2, "%sUBO[%u]:\n", levels[level + 1], desc_idx);
+            dump_domain(contents, 2, level + 2, "A6XX_UBO");
+
+            printl(2, "%sSTORAGE/TEXEL/IMAGE[%u]:\n", levels[level + 1], desc_idx);
+            dump_tex_const(contents, 1, level);
+
+            printl(2, "%sSAMPLER[%u]:\n", levels[level + 1], desc_idx);
+            dump_tex_samp(contents, STATE_SRC_BINDLESS, 1, level);
+         }
+         contents += 16;
+      }
+   }
+}
+
 static void
 cp_load_state(uint32_t *dwords, uint32_t sizedwords, int level)
 {
@@ -1704,9 +1755,16 @@ cp_load_state(uint32_t *dwords, uint32_t sizedwords, int level)
          // TODO probably similar on a4xx..
          if (options->info->chip == 5)
             dump_domain(uboconst, 2, level + 2, "A5XX_UBO");
-         else if (options->info->chip == 6)
+         else if (options->info->chip >= 6)
             dump_domain(uboconst, 2, level + 2, "A6XX_UBO");
          dump_hex(uboconst, 2, level + 1);
+         if (options->dump_textures) {
+            uint64_t addr =
+               (((uint64_t)uboconst[1] & 0x1ffff) << 32) | uboconst[0];
+            /* Size encoded in descriptor is in units of vec4: */
+            unsigned sizedwords = 4 * (uboconst[1] >> 17);
+            dump_gpuaddr_size(addr, level -2, sizedwords, 3);
+         }
          uboconst += src == STATE_SRC_BINDLESS ? 16 : 2;
       }
       break;
@@ -1906,7 +1964,7 @@ cp_event_write(uint32_t *dwords, uint32_t sizedwords, int level)
    if (name && (options->info->chip > 5)) {
       char eventname[64];
       snprintf(eventname, sizeof(eventname), "EVENT:%s", name);
-      if (!strcmp(name, "BLIT")) {
+      if (!strcmp(name, "BLIT") || !strcmp(name, "LRZ_CLEAR")) {
          do_query(eventname, 0);
          print_mode(level);
          dump_register_summary(level);
@@ -2045,8 +2103,10 @@ cp_draw_indx(uint32_t *dwords, uint32_t sizedwords, int level)
    }
 
    /* don't bother dumping registers for the dummy draw_indx's.. */
-   if (num_indices > 0)
+   if (num_indices > 0) {
+      dump_bindless_descriptors(false, level);
       dump_register_summary(level);
+   }
 
    needs_wfi = true;
 }
@@ -2087,8 +2147,10 @@ cp_draw_indx_2(uint32_t *dwords, uint32_t sizedwords, int level)
    }
 
    /* don't bother dumping registers for the dummy draw_indx's.. */
-   if (num_indices > 0)
+   if (num_indices > 0) {
+      dump_bindless_descriptors(false, level);
       dump_register_summary(level);
+   }
 }
 
 static void
@@ -2101,8 +2163,10 @@ cp_draw_indx_offset(uint32_t *dwords, uint32_t sizedwords, int level)
    print_mode(level);
 
    /* don't bother dumping registers for the dummy draw_indx's.. */
-   if (num_indices > 0)
+   if (num_indices > 0) {
+      dump_bindless_descriptors(false, level);
       dump_register_summary(level);
+   }
 }
 
 static void
@@ -2126,6 +2190,7 @@ cp_draw_indx_indirect(uint32_t *dwords, uint32_t sizedwords, int level)
       addr = dwords[3];
    dump_gpuaddr_size(addr, level, 0x10, 2);
 
+   dump_bindless_descriptors(false, level);
    dump_register_summary(level);
 }
 
@@ -2141,6 +2206,7 @@ cp_draw_indirect(uint32_t *dwords, uint32_t sizedwords, int level)
    addr = (((uint64_t)dwords[2] & 0x1ffff) << 32) | dwords[1];
    dump_gpuaddr_size(addr, level, 0x10, 2);
 
+   dump_bindless_descriptors(false, level);
    dump_register_summary(level);
 }
 
@@ -2200,6 +2266,7 @@ cp_draw_indirect_multi(uint32_t *dwords, uint32_t sizedwords, int level)
       }
    }
 
+   dump_bindless_descriptors(false, level);
    dump_register_summary(level);
 }
 
@@ -2211,6 +2278,7 @@ cp_draw_auto(uint32_t *dwords, uint32_t sizedwords, int level)
    do_query(rnn_enumname(rnn, "pc_di_primtype", prim_type), 0);
    print_mode(level);
 
+   dump_bindless_descriptors(false, level);
    dump_register_summary(level);
 }
 
@@ -2336,10 +2404,11 @@ cp_indirect(uint32_t *dwords, uint32_t sizedwords, int level)
        * executed but never returns.  Account for this by checking if
        * the IB returned:
        */
-      highlight_gpuaddr(gpuaddr(dwords));
+      highlight_addr(dwords);
 
       ib++;
       ibs[ib].base = ibaddr;
+      ibs[ib].host_base = ptr;
       ibs[ib].size = ibsize;
 
       dump_commands(ptr, ibsize, level);
@@ -2371,7 +2440,7 @@ cp_start_bin(uint32_t *dwords, uint32_t sizedwords, int level)
        * executed but never returns.  Account for this by checking if
        * the IB returned:
        */
-      highlight_gpuaddr(gpuaddr(&dwords[5]));
+      highlight_addr(&dwords[5]);
 
       /* TODO: we should duplicate the body of the loop after each bin, so
        * that draws get the correct state. We should also figure out if there
@@ -2381,6 +2450,7 @@ cp_start_bin(uint32_t *dwords, uint32_t sizedwords, int level)
       ib++;
       for (uint32_t i = 0; i < loopcount; i++) {
          ibs[ib].base = ibaddr;
+         ibs[ib].host_base = ptr;
          ibs[ib].size = ibsize;
          printl(3, "%sbin %u\n", levels[level], i);
          dump_commands(ptr, ibsize, level);
@@ -2415,11 +2485,12 @@ cp_fixed_stride_draw_table(uint32_t *dwords, uint32_t sizedwords, int level)
        * executed but never returns.  Account for this by checking if
        * the IB returned:
        */
-      highlight_gpuaddr(gpuaddr(&dwords[5]));
+      highlight_addr(&dwords[5]);
 
       ib++;
       for (uint32_t i = 0; i < loopcount; i++) {
          ibs[ib].base = ibaddr;
+         ibs[ib].host_base = ptr;
          ibs[ib].size = ibsize;
          printl(3, "%sdraw %u\n", levels[level], i);
          dump_commands(ptr, ibsize, level);
@@ -2637,6 +2708,7 @@ static void
 cp_exec_cs(uint32_t *dwords, uint32_t sizedwords, int level)
 {
    do_query("compute", 0);
+   dump_bindless_descriptors(true, level);
    dump_register_summary(level);
 }
 
@@ -2655,6 +2727,7 @@ cp_exec_cs_indirect(uint32_t *dwords, uint32_t sizedwords, int level)
    dump_gpuaddr_size(addr, level, 0x10, 2);
 
    do_query("compute", 0);
+   dump_bindless_descriptors(true, level);
    dump_register_summary(level);
 }
 
@@ -2673,11 +2746,11 @@ cp_set_marker(uint32_t *dwords, uint32_t sizedwords, int level)
 
    render_mode = mode;
 
-   if (!strcmp(render_mode, "RM6_BINNING")) {
+   if (!strcmp(render_mode, "RM6_BIN_VISIBILITY")) {
       enable_mask = MODE_BINNING;
-   } else if (!strcmp(render_mode, "RM6_GMEM")) {
+   } else if (!strcmp(render_mode, "RM6_BIN_RENDER_START")) {
       enable_mask = MODE_GMEM;
-   } else if (!strcmp(render_mode, "RM6_BYPASS")) {
+   } else if (!strcmp(render_mode, "RM6_DIRECT_RENDER")) {
       enable_mask = MODE_BYPASS;
    }
 }
@@ -2861,7 +2934,7 @@ cp_reg_write(uint32_t *dwords, uint32_t sizedwords, int level)
 }
 
 static void
-cp_set_ctxswitch_ib(uint32_t *dwords, uint32_t sizedwords, int level)
+cp_set_amble(uint32_t *dwords, uint32_t sizedwords, int level)
 {
    uint64_t addr;
    uint32_t size = dwords[2] & 0xffff;
@@ -2946,7 +3019,7 @@ static const struct type3_op {
    CP(REG_WRITE, cp_reg_write),
    CP(DRAW_AUTO, cp_draw_auto, {.load_all_groups = true}),
 
-   CP(SET_CTXSWITCH_IB, cp_set_ctxswitch_ib),
+   CP(SET_AMBLE, cp_set_amble),
 
    CP(START_BIN, cp_start_bin),
 
