@@ -2,43 +2,31 @@
  * Copyright © 2024 Collabora Ltd. and Red Hat Inc.
  * SPDX-License-Identifier: MIT
  */
-#ifndef NVK_DMA_QUEUE_H
-#define NVK_DMA_QUEUE_H 1
+#ifndef NVK_UPLOAD_QUEUE_H
+#define NVK_UPLOAD_QUEUE_H 1
 
-#include "nvk_private.h"
+#include "nvk_mem_stream.h"
 
-#include "util/list.h"
+#include "nv_push.h"
 #include "util/simple_mtx.h"
 
 struct nvk_device;
-struct nvk_upload_mem;
-struct nvkmd_mem;
 struct nvkmd_ctx;
-struct vk_sync;
 
 struct nvk_upload_queue {
    simple_mtx_t mutex;
 
    struct nvkmd_ctx *ctx;
-
-   struct vk_sync *sync;
+   struct nvk_mem_stream stream;
    uint64_t last_time_point;
 
-   struct nvk_upload_mem *mem;
-
-   /* We grow the buffer from both ends.  Pushbuf data goes at the start of
-    * the buffer and upload data at the tail.
-    */
-   uint32_t mem_push_start;
-   uint32_t mem_push_end;
-   uint32_t mem_data_start;
-
-   /* list of nvk_upload_mem */
-   struct list_head recycle;
+   uint32_t push_data[4096];
+   struct nv_push push;
 };
 
 VkResult nvk_upload_queue_init(struct nvk_device *dev,
                                struct nvk_upload_queue *queue);
+
 void nvk_upload_queue_finish(struct nvk_device *dev,
                              struct nvk_upload_queue *queue);
 
@@ -58,4 +46,4 @@ VkResult nvk_upload_queue_fill(struct nvk_device *dev,
                                struct nvk_upload_queue *queue,
                                uint64_t dst_addr, uint32_t data, size_t size);
 
-#endif /* NVK_DMA_QUEUE_H */
+#endif /* NVK_UPLOAD_QUEUE_H */
